#include "LeoArg.hh"
#include "StringUtils.hh"

using std::string;
/*
bool isnum( const std::string &s );
void toupper( std::string &s );
std::strstream& operator >> ( std::strstream& in, bool& w );
*/

LeoArg::Option::Option( string long_option, string short_option )
  : long_option( long_option ), 
    short_option( short_option )
{
  bool_value = false;
  int_num = 0;
  double_num = 0.0;
  str = "";
  flag = false;
}

LeoArg::LeoArg( Setting& setting )
  : setting( setting )
{
  correct_setting();
   
  for( OptionListIt it = setting.option_list.begin();
       it != setting.option_list.end(); it++ )
    parse_option( *it );
}

LeoArg::LeoArg( int argc, char **argv, string long_option_switch, string short_option_switch )
  :  setting( argc, argv, long_option_switch, short_option_switch )
{
  setting.short_option_string( false );

  correct_setting();

  for( OptionListIt it = setting.option_list.begin();
       it != setting.option_list.end(); it++ )
    parse_option( *it );
}

bool LeoArg::parse_option( Option& option ) const
{
  bool found = false;
  int i;
  for( i=1; i < setting.argc; i++ )
    {
      string av = setting.argv[i];
      if( av == setting.long_option_switch + option.long_option ||
	  av == setting.short_option_switch + option.short_option )
	{
	  found = true;
	  break;
	}
    }

  if( !found )
    {
      if( setting.sos )
	{
	  int s;
	  if( s = has_sos() )
	    {
	      if( is_in_sos( option ) )
		found = true;
	    }
	}
    }

  if( found )
    {
      // find the next option
      int num = 0;
      int j;
      for( j = i + 1; j < setting.argc; j++ )
	{
	  string av = setting.argv[j];
	  if( av.find( setting.long_option_switch ) == 0 ||
	      av.find( setting.short_option_switch ) == 0 )
	    break;
	}

      num = j - i;

      option.flag = true; // the option was found

      if( num > 1 )
	{
	  option.bool_value = StringUtils::s2x<bool>(setting.argv[i+1]);
	  option.int_num = StringUtils::s2x<int>(setting.argv[i+1]);
	  option.double_num = StringUtils::s2x<double>(setting.argv[i+1]);
	  option.str = setting.argv[i+1];
	  
	  for( int j = i+1; j < i + num; j++ )
	    option.arglist.push_back( setting.argv[j] );
	}
      return true;
    }
   
  return false;
}

void LeoArg::correct_option( Option& option ) const
{
  StringUtils::strip( option.long_option );
  StringUtils::strip( option.short_option );

  if( option.long_option.empty() )
    option.long_option = option.short_option;

  if( option.short_option.empty() )
    option.short_option = option.long_option;

  if( option.short_option.empty() && option.long_option.empty() )
    throw InvalidOption( option, "(long_option == \"\") && (short_option == \"\")" );
}

void LeoArg::correct_setting()
{
  if( setting.argc <= 0 ) 
    throw InvalidSetting( "argc <= 0" );

  StringUtils::strip( setting.long_option_switch );
  StringUtils::strip( setting.short_option_switch );

  if( setting.long_option_switch.empty() )
    setting.long_option_switch = setting.short_option_switch;

  if( setting.short_option_switch.empty() )
    setting.short_option_switch = setting.long_option_switch;

  if( setting.short_option_switch.empty() && setting.long_option_switch.empty() )
    throw InvalidSetting( "short_option_switch == \"\" && long_option_switch == \"\"" );

  for( OptionListIt it = setting.option_list.begin(); 
       it != setting.option_list.end(); it++ )
    correct_option( *it );
}

bool LeoArg::get_option( Option& option )
{
  correct_option( option );

  if( find_option( option ) )
    return true;

  setting.add( option );

  return parse_option( option );
}

bool LeoArg::find_option( Option& option ) const
{
  for( OptionListIt it = setting.option_list.begin();
       it != setting.option_list.end(); it++ )
    {
      if( it->long_option == option.long_option )
	{
	  option = *it;
	  return true;
	}
    }
  return false;
}


bool LeoArg::get_first( string& ret_value ) const
{
  if(  has_parameters() )
    {
      string av = setting.argv[1];
      if( av.find( setting.long_option_switch ) == 0 ||
	  av.find( setting.short_option_switch ) == 0 )
	return false;

      ret_value = av;
      return true;
    }
  return false;
}


bool LeoArg::get_first( ArgList& arglist ) const
{
  if(  has_parameters() )
    {
      for( int i = 1; i < setting.argc; i++ )
	{
	  string av = setting.argv[i];
	  if( av.find( setting.long_option_switch ) == 0 ||
	      av.find( setting.short_option_switch ) == 0 )
	    break;
	  arglist.push_back( av );
	}

      return true;
    }
  return false;
}


bool LeoArg::get_last( string& ret_value ) const
{
  if(  has_parameters() )
    {
      string av = setting.argv[setting.argc - 1];
      if( av.find( setting.long_option_switch ) == 0 ||
	  av.find( setting.short_option_switch ) == 0 )
	return false;

      ret_value = av;
      return true;
    }
  return false;
}


bool LeoArg::get_last( ArgList& arglist ) const
{
  if(  has_parameters() )
    {
      for( int i = setting.argc - 1; i > 1; i-- )
	{
	  string av = setting.argv[i];
	  if( av.find( setting.long_option_switch ) == 0 ||
	      av.find( setting.short_option_switch ) == 0 )
	    break;
	  arglist.push_back( av );
	}

      return true;
    }
  return false;
}

bool LeoArg::is_in_sos( const Option& option ) const
{
  if( option.short_option.size() != 1 )
    return false; // only one character options are allowed

  int i;

  if( i = has_sos() )
    {
      string arg = setting.argv[i];
      arg = arg.substr( 1 ); // eleminate option symbol

      if( arg.find( option.short_option ) != string::npos )
	return true;
    }

  return false;
}

/** If it finds a short option string it returns the position 
    of it (argv[i]). If no sos was found it returns zero */
int LeoArg::has_sos() const
{
  if( !has_parameters() )
    return 0;

  for( int i = 1; i < setting.argc; i++ )
    {
      string av = setting.argv[i];
      if( av.find( setting.short_option_switch ) == 0 && 
	  av.find( setting.long_option_switch ) != 0 )
	{
	  Option o( "#__LEOARG__#", av.substr( 1 ) );
	  if( !find_option( o ) )
	    return i;
	}
    }
  return 0;
}

extern "C" {
  char leoarg() { return 1; }
  char leoarg_version_2_2() { return 1; }
  char leoarg_version_2_2_1() { return 1; }
}
